module Day13Lib
  ( Value (..),
    parseList,
    parseNumber,
    --
    processInput1,
    processInput2,
  )
where

-- import Data.Either
import Data.List
import Data.List.Split (splitOn)
import Debug.Trace
import Text.Parsec

data Value = Num Int | List [Value] deriving (Show, Eq)

parseList :: Parsec String () Value
parseList = do
  char '['
  ls <- (parseNumber <|> parseList) `sepBy` char ','
  char ']'
  return $ List ls

parseNumber :: Parsec String () Value
parseNumber = do
  ns <- try . many1 $ digit
  return $ Num (read ns)

cmp :: Value -> Value -> Ordering
cmp (Num x) (Num y) = compare x y
cmp x@(Num _) y@(List _) = cmp (List [x]) y
cmp x@(List _) y@(Num _) = cmp x (List [y])
cmp (List (x : xs)) (List (y : ys)) = case cmp x y of
  EQ -> cmp (List xs) (List ys)
  o -> o
cmp (List []) (List (y : ys)) = LT
cmp (List (x : xs)) (List []) = GT
cmp (List []) (List []) = EQ

runParse :: String -> Value
runParse s = case parse parseList "" s of
  Left e -> error $ "parse error: " ++ show e
  Right v -> v

-- ####################

processInput1 :: String -> Int
processInput1 =
  sum
    . zipWith (\x y -> if y == LT then x else 0) [1 ..]
    . map
      ( (\[x, y] -> cmp x y)
          . map runParse
          . lines
      )
    . splitOn "\n\n"

processInput2 :: String -> Int
processInput2 =
  product
    . zipWith (\x y -> if y `elem` dividers then x else 1) [1 ..]
    . sortBy cmp
    . (++) dividers
    . (concatMap (map runParse . lines) . splitOn "\n\n")

dividers = [runParse "[[2]]", runParse "[[6]]"]